searchNodes=[{"ref":"Rabbit.Connection.html","title":"Rabbit.Connection","type":"behaviour","doc":"A RabbitMQ connection process. Connections form the basis of any application that is working with RabbitMQ. A connection module is needed by all the other modules included with Rabbit. They wrap around the standard AMQP.Connection and provide the following benefits: Durability during connection failures through use of expotential backoff. Subscriptions that assist connection status monitoring. Easy runtime setup through an init/2 callback. Example # Connection module defmodule MyConnection do use Rabbit.Connection def start_link(opts \\\\ []) do Rabbit.Connection.start_link(__MODULE__, opts, name: __MODULE__) end # Callbacks @impl Rabbit.Connection def init(_type, opts) do # Perform any runtime configuration {:ok, opts} end end # Start the connection MyConnection.start_link() # Subscribe to the connection Rabbit.Connection.subscribe(MyConnection) receive do {:connected, connection} -&gt; &quot;hello&quot; end # Stop the connection Rabbit.Connection.stop(MyConnection) receive do {:disconnected, reason} -&gt; &quot;bye&quot; end"},{"ref":"Rabbit.Connection.html#alive?/1","title":"Rabbit.Connection.alive?/1","type":"function","doc":"Checks whether a connection is alive."},{"ref":"Rabbit.Connection.html#fetch/1","title":"Rabbit.Connection.fetch/1","type":"function","doc":"Fetches the raw AMQP.Connection struct from the process."},{"ref":"Rabbit.Connection.html#c:init/2","title":"Rabbit.Connection.init/2","type":"callback","doc":"A callback executed when the connection is started. Returning {:ok, opts} - where opts is a keyword list of option/0 will, cause start_link/3 to return {:ok, pid} and the process to enter its loop. Returning :ignore will cause start_link/3 to return :ignore and the process will exit normally without entering the loop."},{"ref":"Rabbit.Connection.html#start_link/3","title":"Rabbit.Connection.start_link/3","type":"function","doc":"Starts a connection process. Options :uri - The connection URI. This takes priority over other connection attributes. :username - The name of a user registered with the broker - defaults to &quot;guest&quot;. :password - The password of user - defaults to &quot;guest. :virtual_host - The name of a virtual host in the broker - defaults to &quot;/&quot;. :host - The hostname of the broker - defaults to &quot;localhost&quot;. :port - The port the broker is listening on - defaults to 5672. :channel_max - The channel_max handshake parameter - defaults to 0. :frame_max - The frame_max handshake parameter - defaults to 0. :heartbeat - The hearbeat interval in seconds - defaults to 10. :connection_timeout - The connection timeout in milliseconds - defaults to 50000. :retry_backoff - The amount of time in milliseconds to add between connection retry attempts - defaults to 1_000. :retry_max_delay - The max amount of time in milliseconds to be used between connection attempts - defaults to 5_000. :ssl_options - Enable SSL by setting the location to cert files - defaults to :none. :client_properties - A list of extra client properties to be sent to the server - defaults to []. :socket_options - Extra socket options. These are appended to the default options. See http://www.erlang.org/doc/man/inet.html#setopts-2 and http://www.erlang.org/doc/man/gen_tcp.html#connect-4 for descriptions of the available options. Server Options You can also provide server options - which are simply the same ones available for GenServer.options/0."},{"ref":"Rabbit.Connection.html#stop/1","title":"Rabbit.Connection.stop/1","type":"function","doc":"Stops a connection process."},{"ref":"Rabbit.Connection.html#subscribe/2","title":"Rabbit.Connection.subscribe/2","type":"function","doc":"Subscribes a process to the connection. A subscribed process can receive the following messages: {:connected, connection} - where connection is an AMQP.Connection struct. During the subscription process, if the connection is alive, this message will immediately be sent. If the connection goes down, and manages to reconnect, this message will be sent. {:disconnected, reason} - where reason can be any value. If the connection goes down, all subscribing processes are sent this message. The connection process will then go through an exponential backoff period until connection is achieved again."},{"ref":"Rabbit.Connection.html#t:option/0","title":"Rabbit.Connection.option/0","type":"type","doc":""},{"ref":"Rabbit.Connection.html#t:options/0","title":"Rabbit.Connection.options/0","type":"type","doc":""},{"ref":"Rabbit.Connection.html#t:t/0","title":"Rabbit.Connection.t/0","type":"type","doc":""},{"ref":"Rabbit.Connection.html#t:uri/0","title":"Rabbit.Connection.uri/0","type":"type","doc":""},{"ref":"Rabbit.Consumer.html","title":"Rabbit.Consumer","type":"behaviour","doc":"A RabbitMQ consumer process. Consumers are the &quot;workers&quot; of your application. They wrap around the standard AMQP.Channel and provide the following benefits: Durability during connection and channel failures through use of expotential backoff. Easy runtime setup through the init/2 and handle_setup/2 callbacks. Automatic acknowledgements based on the return value of the handle_message/1 callback. Ability to handle exceptions through the handle_error/1 callback. Each message is executed within its own supervised task. Automatic payload decoding based on available serializers and message content type. Example # This is a connection defmodule MyConnection do use Rabbit.Connection def start_link(opts \\\\ []) do Rabbit.Connection.start_link(__MODULE__, opts, name: __MODULE__) end # Callbacks @impl Rabbit.Connection def init(:connection, opts) do # Perform any runtime configuration {:ok, opts} end end # This is a consumer defmodule MyConsumer do use Rabbit.Consumer def start_link(opts \\\\ []) do Rabbit.Consumer.start_link(__MODULE__, opts, name: __MODULE__) end # Callbacks @impl Rabbit.Consumer def init(_type, opts) do # Perform any runtime configuration {:ok, opts} end @impl Rabbit.Consumer def handle_setup(channel, queue) do # Optional callback to perform exchange or queue setup AMQP.Queue.declare(channel, queue) :ok end @impl Rabbit.Consumer def handle_message(message) do # Handle consumed messages {:ack, message} end @impl Rabbit.Consumer def handle_error(message) do # Handle errors that occur within handle_message/1 {:nack, message} end end # Start the connection MyConnection.start_link() # Start the consumer MyConsumer.start_link(connection: MyConnection, queue: &quot;my_queue&quot;, prefetch_count: 20) Serializers When a message is consumed, its content type is compared to the list of available serializers. If a serializer matches the content type, the message will be automatically decoded. You can find out more about serializers at Rabbit.Serializer."},{"ref":"Rabbit.Consumer.html#c:handle_error/1","title":"Rabbit.Consumer.handle_error/1","type":"callback","doc":"A callback executed to handle message exceptions. If the original handle_message/1 callback raises an error, this callback will be called with the message - but with the :error_reason and :error_stack fields filled. You may choose to return the same values as handle_message/1."},{"ref":"Rabbit.Consumer.html#c:handle_message/1","title":"Rabbit.Consumer.handle_message/1","type":"callback","doc":"A callback executed to handle message consumption. The callback is provided a Rabbit.Message struct. You may find more information about the message structure within its own documentation. The message may be automatically decoded based on the content type and available serializers. You may choose to ack, nack, or reject the message based on the return value. {:ack, message} - will awknowledge the message. {:ack, message, options} - will awknowledge the message with options. {:nack, message} - will negative awknowledge the message. {:nack, message, options} - will negative awknowledge the message with options. {:reject, message} - will reject the message. {:reject, message, options} - will reject the message with options. If you dont return one of these values - nothing will be done. This means you will need to manually ack, nack or reject the message if required. Please see the Rabbit.Message module for more information."},{"ref":"Rabbit.Consumer.html#c:handle_setup/2","title":"Rabbit.Consumer.handle_setup/2","type":"callback","doc":"An optional callback executed after the channel is open, but before consumption. The callback is called with an AMQP.Channel, as well as the queue that will be consumed. At the most basic, you may want to declare the queue to ensure its available. This will be entirely application dependent though. def handle_setup(channel, queue) do AMQP.Queue.declare(channel, queue) :ok end The callback must return an :ok atom - otherise it will be marked as failed, and the consumer will attempt to go through the connection setup process again. Alternatively, you could use a Rabbit.Initializer process to perform this setup work. Please see its docs for more information."},{"ref":"Rabbit.Consumer.html#c:init/2","title":"Rabbit.Consumer.init/2","type":"callback","doc":"A callback executed when the consumer is started. Returning {:ok, opts} - where opts is a keyword list of t:option() will, cause start_link/3 to return {:ok, pid} and the process to enter its loop. Returning :ignore will cause start_link/3 to return :ignore and the process will exit normally without entering the loop"},{"ref":"Rabbit.Consumer.html#start_link/3","title":"Rabbit.Consumer.start_link/3","type":"function","doc":"Starts a consumer process. Options :connection - A Rabbit.Connection process. :queue - The queue to consume messages from. :prefetch_count - The basic unit of concurrency for a given consumer - defaults to 0. :prefetch_size - The prefetch window size in octets - defaults to 0, meaning no specific limit. :consumer_tag - The identifier of the consumer. If empty, one will be generared by the server. :no_local - A boolean representing whether messages should not be sent to the same connection that published them - defaults to false. :no_ack - A boolean representing whether acknowledgements are not required for messages - defaults to false. :exclusive - A boolean representing whether only this consumer can access the queue - defaults to false. :no_wait - A boolean representing whether the server should not respond to methods - defaults to false. :arguments - A set of arguments for the consume. Server Options You can also provide server options - which are simply the same ones available for GenServer.options/0."},{"ref":"Rabbit.Consumer.html#stop/1","title":"Rabbit.Consumer.stop/1","type":"function","doc":"Stops a consumer process."},{"ref":"Rabbit.Consumer.html#t:action_options/0","title":"Rabbit.Consumer.action_options/0","type":"type","doc":""},{"ref":"Rabbit.Consumer.html#t:delivery_tag/0","title":"Rabbit.Consumer.delivery_tag/0","type":"type","doc":""},{"ref":"Rabbit.Consumer.html#t:option/0","title":"Rabbit.Consumer.option/0","type":"type","doc":""},{"ref":"Rabbit.Consumer.html#t:options/0","title":"Rabbit.Consumer.options/0","type":"type","doc":""},{"ref":"Rabbit.Consumer.html#t:t/0","title":"Rabbit.Consumer.t/0","type":"type","doc":""},{"ref":"Rabbit.ConsumerSupervisor.html","title":"Rabbit.ConsumerSupervisor","type":"behaviour","doc":"A RabbitMQ consumer supervisor process. This allows starting and supervising multiple Rabbit.Consumer processes with ease. Rather than creating a module for each consumer and implementing repetitive logic - the same callbacks are used across all child consumers. Example # This is a connection defmodule MyConnection do use Rabbit.Connection def start_link(opts \\\\ []) do Rabbit.Connection.start_link(__MODULE__, opts, name: __MODULE__) end # Callbacks @impl Rabbit.Connection def init(:connection, opts) do # Perform any runtime configuration {:ok, opts} end end # This is a consumer supervisor defmodule MyConsumers do use Rabbit.ConsumerSupervisor def start_link(consumers \\\\ []) do Rabbit.ConsumerSupervisor.start_link(__MODULE__, consumers, name: __MODULE__) end # Callbacks @impl Rabbit.ConsumerSupervisor def init(:consumer_supervisor, _consumers) do # Perform any runtime configuration for the supervisor consumers = [ [connection: MyConnection, queue: &quot;my_queue1&quot;, prefetch_count: 5], [connection: MyConnection, queue: &quot;my_queue2&quot;, prefetch_count: 10] ] {:ok, consumers} end def init(:consumer, opts) do # Perform any runtime configuration per consumer {:ok, opts} end @impl Rabbit.ConsumerSupervisor def handle_setup(channel, queue) do # Perform exchange or queue setup per consumer AMQP.Queue.declare(channel, queue) :ok end @impl Rabbit.ConsumerSupervisor def handle_message(message) do # Handle messages per consumer {:ack, message} end @impl Rabbit.ConsumerSupervisor def handle_error(message) do # Handle errors that occur per consumer {:nack, message} end end # Start the connection MyConnection.start_link() # Start the consumers MyConsumers.start_link()"},{"ref":"Rabbit.ConsumerSupervisor.html#c:handle_error/1","title":"Rabbit.ConsumerSupervisor.handle_error/1","type":"callback","doc":"A callback executed by each consumer to handle message exceptions. Please see Rabbit.Consumer.handle_error/1 for more information."},{"ref":"Rabbit.ConsumerSupervisor.html#c:handle_message/1","title":"Rabbit.ConsumerSupervisor.handle_message/1","type":"callback","doc":"A callback executed by each consumer to handle message consumption. Please see Rabbit.Consumer.handle_message/1 for more information."},{"ref":"Rabbit.ConsumerSupervisor.html#c:handle_setup/2","title":"Rabbit.ConsumerSupervisor.handle_setup/2","type":"callback","doc":"A callback executed by each consumer after the channel is open, but before consumption. Please see Rabbit.Consumer.handle_setup/2 for more information."},{"ref":"Rabbit.ConsumerSupervisor.html#c:init/2","title":"Rabbit.ConsumerSupervisor.init/2","type":"callback","doc":"A callback executed by each component of the consumer supervisor. Two versions of the callback must be created. One for the supervisor, and one for the consumers. The first argument differentiates the callback. # Initialize the supervisor def init(:consumer_supervisor, consumers) do {:ok, consumers} end # Initialize a single consumer def init(:consumer, opts) do {:ok, opts} end Returning {:ok, consumers} - where consumers is a list of Rabbit.Consumer.options/0 will, cause start_link/3 to return {:ok, pid} and the supervisor to enter its loop. Returning {:ok, opts} - where opts is a keyword list of Rabbit.Consumer.option/0 will, cause start_link/3 to return {:ok, pid} and the consumer to enter its loop. Returning :ignore will cause start_link/3 to return :ignore and the process will exit normally without entering the loop."},{"ref":"Rabbit.ConsumerSupervisor.html#start_link/3","title":"Rabbit.ConsumerSupervisor.start_link/3","type":"function","doc":"Starts a consumer supervisor process. The list of consumers should represent a list of Rabbit.Consumer.options/0. Server Options You can also provide server options - which are simply the same ones available for GenServer.options/0."},{"ref":"Rabbit.ConsumerSupervisor.html#t:consumers/0","title":"Rabbit.ConsumerSupervisor.consumers/0","type":"type","doc":""},{"ref":"Rabbit.ConsumerSupervisor.html#t:t/0","title":"Rabbit.ConsumerSupervisor.t/0","type":"type","doc":""},{"ref":"Rabbit.Initializer.html","title":"Rabbit.Initializer","type":"behaviour","doc":"A RabbitMQ initializer process. This is a blocking process that can be used to declare exchanges, queues, and bindings. It will perform any setup required by your application. It should be added to your supervision tree before any producers or consumers. Both Rabbit.Consumer and Rabbit.ConsumerSupervisor have the handle_setup/2 callback, which can be used to perform any queue, exchange or binding work as well. But if you have more complex requirements, this module can be used. Example # This is a connection defmodule MyConnection do use Rabbit.Connection def start_link(opts \\ []) do Rabbit.Connection.start_link(__MODULE__, opts, name: __MODULE__) end # Callbacks @impl Rabbit.Connection def init(:connection, opts) do # Perform any runtime configuration {:ok, opts} end end # This is an initializer defmodule MyInitializer do use Rabbit.Initializer def start_link(opts \\ []) do Rabbit.Initializer.start_link(__MODULE__, opts, name: __MODULE__) end # Callbacks @impl Rabbit.Initializer def init(_type, opts) do # Perform any runtime configuration {:ok, opts} end end # Start the connection MyConnection.start_link() # Start the initializer MyInitializer.start_link( connection: MyConnection, exchanges: [ [name: &quot;my_exchange_1&quot;], [name: &quot;my_exchange_2&quot;, type: :fanout], ], queues: [ [name: &quot;my_queue_1&quot;], [name: &quot;my_queue_2&quot;, durable: true], ], bindings: [ [type: :queue, source: &quot;my_exchange_1&quot;, destination: &quot;my_queue_1&quot;], [type: :exchange, source: &quot;my_exchange_2&quot;, destination: &quot;my_exchange_1&quot;], ] )"},{"ref":"Rabbit.Initializer.html#c:init/2","title":"Rabbit.Initializer.init/2","type":"callback","doc":"A callback executed when the initializer is started. Returning {:ok, opts} - where opts is a keyword list of option/0 will cause start_link/3 to return {:ok, pid} and the process to enter its loop. Returning :ignore will cause start_link/3 to return :ignore and the process will exit normally without entering the loop"},{"ref":"Rabbit.Initializer.html#start_link/3","title":"Rabbit.Initializer.start_link/3","type":"function","doc":"Starts an initializer process. Options :connection - A Rabbit.Connection process. :exchanges - A list of exchanges to declare. Please see Exchanges. :queues - A list of queues to declare. Please see Queues. :bindings - A list of bindings to declare. Please see Bindings. :retry_sleep - The amount of time in milliseconds to sleep between attempts to fetch a connection from the connection process - defaults to 100. :retry_max - The max amount of connection retries that will be attempted before returning an error - defaults to 25. Exchanges Declaring exchanges is done by providing a list of keyword options. The options include: :name - The name of the exchange. :type - The type of the exchange - one of :direct, :fanout, :topic, :match or :headers - defaults to :direct. :durable - Whether the exchange is durable across broker restarts - defaults to false. :auto_delete - Deletes the exchange once all queues unbind from it - defaults to false. :passive - Returns an error if the exchange does not already exist - defaults to false. :internal - If set, the exchange may not be used directly by publishers, but only when bound to other exchanges. Internal exchanges are used to construct wiring that is not visible to applications - defaults to false. Below is an example of exchange options: [ [name: &quot;my_exchange_1&quot;], [name: &quot;my_exchange_2&quot;, type: :fanout, durable: true], ] Queues Declaring queues is done by providing a list of keyword options. The options include: :name - The name of the queue. :durable - Whether the queue is durable across broker restarts - defaults to false. :auto_delete - Deletes the queue once all consumers disconnect - defaults to false. :passive - Returns an error if the queue does not already exist - defaults to false. :exclusive - If set, only one consumer can consume from the queue - defaults to false. Below is an example of queue options: [ [name: &quot;my_queue_1&quot;], [name: &quot;my_queue_2&quot;, durable: true], ] Bindings Declaring bindings is done by providing a list of keyword options. The options include: :type - The type of the destination - one of :exchange or :queue. :source - The source of the binding. :destination - The destination of the binding. :routing_key - The routing key of the binding. Below is an example of binding options: [ [type: :queue, source: &quot;my_exchange_1&quot;, destination: &quot;my_queue_1&quot;], [type: :exchange, source: &quot;my_exchange_2&quot;, destination: &quot;my_exchange_1&quot;] ] Server Options You can also provide server options - which are simply the same ones available for GenServer.options/0."},{"ref":"Rabbit.Initializer.html#t:binding/0","title":"Rabbit.Initializer.binding/0","type":"type","doc":""},{"ref":"Rabbit.Initializer.html#t:exchange/0","title":"Rabbit.Initializer.exchange/0","type":"type","doc":""},{"ref":"Rabbit.Initializer.html#t:option/0","title":"Rabbit.Initializer.option/0","type":"type","doc":""},{"ref":"Rabbit.Initializer.html#t:options/0","title":"Rabbit.Initializer.options/0","type":"type","doc":""},{"ref":"Rabbit.Initializer.html#t:queue/0","title":"Rabbit.Initializer.queue/0","type":"type","doc":""},{"ref":"Rabbit.Initializer.html#t:t/0","title":"Rabbit.Initializer.t/0","type":"type","doc":""},{"ref":"Rabbit.Message.html","title":"Rabbit.Message","type":"module","doc":"A message consumed by a Rabbit.Consumer. After starting a consumer, any message passed to the Rabbit.Consumer.handle_message/1 callback will be wrapped in a messsage struct. The struct has the following fields: :consumer - The PID of the consumer process. :module - The module of the consumer process. :channel - The AMQP.Channel being used by the consumer. :payload - The raw payload of the message. :decoded_payload - If the message has a content type - this will be the payload decoded using the applicable serializer. :meta - The metadata sent when publishing or set by the broker. :error_reason - The reason for any error that occurs during the message handling callback. :error_stack - The stacktrace that might accompany the error."},{"ref":"Rabbit.Message.html#ack/2","title":"Rabbit.Message.ack/2","type":"function","doc":"Awknowledges a message. Options :multiple - If true, all messages up to the one specified by its delivery_tag are considered acknowledged by the server."},{"ref":"Rabbit.Message.html#nack/2","title":"Rabbit.Message.nack/2","type":"function","doc":"Negative awknowledges a message. Options :multiple - If true, all messages up to the one specified by it delivery_tag are considered acknowledged by the server. :requeue - If true, the message will be returned to the queue and redelivered to the next available consumer."},{"ref":"Rabbit.Message.html#new/5","title":"Rabbit.Message.new/5","type":"function","doc":"Creates a new message struct."},{"ref":"Rabbit.Message.html#reject/2","title":"Rabbit.Message.reject/2","type":"function","doc":"Rejects a message. Options :requeue - If true, the message will be returned to the queue and redelivered to the next available consumer."},{"ref":"Rabbit.Message.html#t:t/0","title":"Rabbit.Message.t/0","type":"type","doc":""},{"ref":"Rabbit.Producer.html","title":"Rabbit.Producer","type":"behaviour","doc":"A RabbitMQ producer process. Producers are needed to publish any messages to RabbitMQ. They wrap around the standard AMQP.Channel and provide the following benefits: Durability during connection and channel failures through use of expotential backoff. Channel pooling for increased publishing performance. Easy runtime setup through an init/2 and handle_setup/1 callbacks. Simplification of standard publishing options. Automatic payload encoding based on available serializers and message content type. Example # This is a connection defmodule MyConnection do use Rabbit.Connection def start_link(opts \\\\ []) do Rabbit.Connection.start_link(__MODULE__, opts, name: __MODULE__) end # Callbacks @impl Rabbit.Connection def init(_type, opts) do {:ok, opts} end end # This is a producer defmodule MyProducer do use Rabbit.Producer def start_link(opts \\\\ []) do Rabbit.Producer.start_link(__MODULE__, opts, name: __MODULE__) end # Callbacks @impl Rabbit.Producer def init(:producer_pool, opts) do # Perform any runtime configuration for the pool {:ok, opts} end def init(:producer, opts) do # Perform any runtime configuration per producer {:ok, opts} end end # Start the connection MyConnection.start_link() # Start the producer MyProducer.start_link(connection: MyConnection, publish_opts: [content_type: &quot;application/json&quot;]) # Publish a message Rabbit.Producer.publish(MyProducer, &quot;&quot;, &quot;my_queue&quot;, %{foo: &quot;bar&quot;}) Serializers When a message is published, its content type is compared to the list of available serializers. If a serializer matches the content type, the message will be automatically encoded. You can find out more about serializers at Rabbit.Serializer."},{"ref":"Rabbit.Producer.html#c:handle_setup/1","title":"Rabbit.Producer.handle_setup/1","type":"callback","doc":"An optional callback executed after the channel is open. The callback is called with an AMQP.Channel. At the most basic, you may want to declare queues that you will be publishing to. def handle_setup(channel) do AMQP.Queue.declare(channel, &quot;some_queue&quot;) :ok end The callback must return an :ok atom - otherise it will be marked as failed, and the consumer will attempt to go through the connection setup process again. Alternatively, you could use a Rabbit.Initializer process to perform this setup work. Please see its docs for more information."},{"ref":"Rabbit.Producer.html#c:init/2","title":"Rabbit.Producer.init/2","type":"callback","doc":"A callback executed by each component of the producer. Two versions of the callback must be created. One for the pool, and one for the producers. The first argument differentiates the callback. # Initialize the pool def init(:producer_pool, opts) do {:ok, opts} end # Initialize a single producer def init(:producer, opts) do {:ok, opts} end Returning {:ok, opts} - where opts is a keyword list of t:option() will, cause start_link/3 to return {:ok, pid} and the process to enter its loop. Returning :ignore will cause start_link/3 to return :ignore and the process will exit normally without entering the loop"},{"ref":"Rabbit.Producer.html#publish/6","title":"Rabbit.Producer.publish/6","type":"function","doc":"Publishes a message using the provided producer. All publishing options can be provided to the producer during process start. They would then be used as defaults during publishing. Options provided to this function would overwrite any defaults the producer has. Serializers If a content_type is provided as an option - and it matches one of the available serializers, the payload will be automatically encoded using that serializer. For example, we could automatically encode our payload to json if we do the following: Rabbit.Producer.publish(MyProducer, &quot;&quot;, &quot;my_queue&quot;, %{foo: &quot;bar&quot;}, content_type: &quot;application/json&quot;) Please see the documention at Rabbit.Serializer for more information. Options :mandatory - If set, returns an error if the broker can&#39;t route the message to a queue - defaults to false. :immediate - If set, returns an error if the broker can&#39;t deliver te message to a consumer immediately - defaults to false. :content_type - MIME Content type. :content_encoding - MIME Content encoding. :headers - Message headers. Can be used with headers Exchanges. :persistent - If set, uses persistent delivery mode. Messages marked as persistent that are delivered to durable queues will be logged to disk. :correlation_id - Application correlation identifier :priority - Message priority, ranging from 0 to 9. :reply_to - Name of the reply queue. :expiration - How long the message is valid (in milliseconds). :message_id - Message identifier. :timestamp - Timestamp associated with this message (epoch time). :type - Message type as a string. :user_id - Creating user ID. RabbitMQ will validate this against the active connection user. :app_id - Publishing application ID."},{"ref":"Rabbit.Producer.html#start_link/3","title":"Rabbit.Producer.start_link/3","type":"function","doc":"Starts a producer process. Options :connection - A Rabbit.Connection process. :pool_size - The number of processes to create for publishing - defaults to 1. Each process consumes a RabbitMQ channel. :max_overflow - Maximum number of temporary workers created when the pool is empty - defaults to 0. :sync_start - Boolean representing whether to establish the connection and channel syncronously - defaults to true. :sync_start_delay - The amount of time in milliseconds to sleep between sync start attempts - defaults to 50. :sync_start_max - The max amount of sync start attempts that will occur before proceeding with async start - defaults to 100. :publish_opts - Any publish_option/0 that is automatically set as a default option value when calling publish/6. Server Options You can also provide server options - which are simply the same ones available for GenServer.options/0."},{"ref":"Rabbit.Producer.html#stop/1","title":"Rabbit.Producer.stop/1","type":"function","doc":"Stops a producer process."},{"ref":"Rabbit.Producer.html#t:exchange/0","title":"Rabbit.Producer.exchange/0","type":"type","doc":""},{"ref":"Rabbit.Producer.html#t:message/0","title":"Rabbit.Producer.message/0","type":"type","doc":""},{"ref":"Rabbit.Producer.html#t:option/0","title":"Rabbit.Producer.option/0","type":"type","doc":""},{"ref":"Rabbit.Producer.html#t:options/0","title":"Rabbit.Producer.options/0","type":"type","doc":""},{"ref":"Rabbit.Producer.html#t:pool_option/0","title":"Rabbit.Producer.pool_option/0","type":"type","doc":""},{"ref":"Rabbit.Producer.html#t:pool_options/0","title":"Rabbit.Producer.pool_options/0","type":"type","doc":""},{"ref":"Rabbit.Producer.html#t:producer_option/0","title":"Rabbit.Producer.producer_option/0","type":"type","doc":""},{"ref":"Rabbit.Producer.html#t:producer_options/0","title":"Rabbit.Producer.producer_options/0","type":"type","doc":""},{"ref":"Rabbit.Producer.html#t:publish_option/0","title":"Rabbit.Producer.publish_option/0","type":"type","doc":""},{"ref":"Rabbit.Producer.html#t:publish_options/0","title":"Rabbit.Producer.publish_options/0","type":"type","doc":""},{"ref":"Rabbit.Producer.html#t:routing_key/0","title":"Rabbit.Producer.routing_key/0","type":"type","doc":""},{"ref":"Rabbit.Producer.html#t:t/0","title":"Rabbit.Producer.t/0","type":"type","doc":""},{"ref":"Rabbit.Serializer.html","title":"Rabbit.Serializer","type":"behaviour","doc":"A behaviour to implement serializers. To create a serializer, you just need to implement the encode/1 and decode/1 callbacks, Example defmodule MySerializer do @behaviour Rabbit.Serializer @impl true def encode(data) do # Create a binary from data end @impl true def decode(binary) do # Create data from a binary end end Default Serializers By default, Rabbit comes with serializers for the following content types: &quot;application/etf&quot; - built in erlang term format. &quot;application/json&quot; - requires the Jason library to be added. You can modify the available serializers through application config: config :rabbit, serializers: %{ &quot;application/custom-type&quot; =&gt; MySerializer }"},{"ref":"Rabbit.Serializer.html#c:decode/1","title":"Rabbit.Serializer.decode/1","type":"callback","doc":"Callback invoked to decode the given binary to data."},{"ref":"Rabbit.Serializer.html#c:encode/1","title":"Rabbit.Serializer.encode/1","type":"callback","doc":"Callback invoked to encode the given data to a binary."},{"ref":"Rabbit.Serializer.html#t:t/0","title":"Rabbit.Serializer.t/0","type":"type","doc":""},{"ref":"readme.html","title":"Rabbit","type":"extras","doc":"Rabbit Rabbit is a set of tools for building applications with RabbitMQ."},{"ref":"readme.html#installation","title":"Rabbit - Installation","type":"extras","doc":"The package can be installed by adding rabbit to your list of dependencies in mix.exs: def deps do [ {:rabbit, &quot;~&gt; 0.6&quot;} ] end"},{"ref":"readme.html#documentation","title":"Rabbit - Documentation","type":"extras","doc":"Please see HexDocs for additional documentation."},{"ref":"readme.html#connections","title":"Rabbit - Connections","type":"extras","doc":"Connections form the basis of any application that is working with RabbitMQ. A connection module is needed by all the other modules included with Rabbit. defmodule MyConnection do use Rabbit.Connection def start_link(opts \\\\ []) do Rabbit.Connection.start_link(__MODULE__, opts, name: __MODULE__) end # Callbacks @impl Rabbit.Connection def init(:connection, opts) do # Perform runtime config uri = System.get_env(&quot;RABBITMQ_URI&quot;) || &quot;amqp://guest:guest@127.0.0.1:5672&quot; opts = Keyword.put(opts, :uri, uri) {:ok, opts} end end MyConnection.start_link()"},{"ref":"readme.html#consumers","title":"Rabbit - Consumers","type":"extras","doc":"Consumers are the &quot;workers&quot; of your application. They must be provided a connection module and queue to consume. Every message recieved is then passed along to your handle_message/1 callback within its own process. You can optionally implement the handle_setup/2 callback to perform any work needed to declare queues/exchanges/bindings. defmodule MyConsumer do use Rabbit.Consumer def start_link(opts \\\\ []) do Rabbit.Consumer.start_link(__MODULE__, opts, name: __MODULE__) end # Callbacks @impl Rabbit.Consumer def init(:consumer, opts) do # Perform runtime config {:ok, opts} end @impl Rabbit.Consumer def handle_setup(channel, queue) do # Optional callback to perform any exchange or queue setup AMQP.Queue.declare(channel, queue) :ok end @impl Rabbit.Consumer def handle_message(message) do # Handle message consumption IO.inspect(message.payload) {:ack, message} end @impl Rabbit.Consumer def handle_error(message) do # Handle message errors {:nack, message} end end MyConsumer.start_link(connection: MyConnection, queue: &quot;my_queue&quot;, prefetch_count: 10)"},{"ref":"readme.html#consumer-supervisors","title":"Rabbit - Consumer Supervisors","type":"extras","doc":"Consumer supervisors provide an easy way to start and supervise multiple consumer processes. Rather than creating a module for each consumer and implementing repetitive logic - the same callbacks are used across all consumers. defmodule MyConsumerSupervisor do use Rabbit.ConsumerSupervisor def start_link(consumers \\\\ []) do Rabbit.ConsumerSupervisor.start_link(__MODULE__, consumers, name: __MODULE__) end # Callbacks @impl Rabbit.ConsumerSupervisor def init(:consumer_supervisor, _consumers) do # Perform runtime config for the consumer supervisor consumers = [ [connection: MyConnection, queue: &quot;my_queue1&quot;, prefetch_count: 5], [connection: MyConnection, queue: &quot;my_queue2&quot;, prefetch_count: 10], ] {:ok, consumers} end def init(:consumer, opts) do # Perform runtime config per consumer {:ok, opts} end @impl Rabbit.ConsumerSupervisor def handle_setup(channel, queue) do # Optional callback to perform any exchange or queue setup per consumer AMQP.Queue.declare(channel, queue) :ok end @impl Rabbit.ConsumerSupervisor def handle_message(message) do # Handle message consumption per consumer IO.inspect(message.payload) {:ack, message} end @impl Rabbit.ConsumerSupervisor def handle_error(message) do # Handle message errors per consumer {:nack, message} end end MyConsumerSupervisor.start_link()"},{"ref":"readme.html#producers","title":"Rabbit - Producers","type":"extras","doc":"In order to publish messages to RabbitMQ, we must create a producer module. They must be provided a connection module. The producer will then automatically create a pool of channels to publish from. You can optionally implement the handle_setup/1 callback to perform any work needed to declare queues/exchanges/bindings. defmodule MyProducer do use Rabbit.Producer def start_link(opts \\\\ []) do Rabbit.Producer.start_link(__MODULE__, opts, name: __MODULE__) end # Callbacks @impl Rabbit.Producer def init(:producer_pool, opts) do # Perform runtime config for the producer pool {:ok, opts} end def init(:producer, opts) do # Perform runtime config per producer {:ok, opts} end end MyProducer.start_link(connection: MyConnection) Rabbit.Producer.publish(MyProducer, &quot;&quot;, &quot;my_queue&quot;, &quot;hello&quot;)"},{"ref":"readme.html#initializers","title":"Rabbit - Initializers","type":"extras","doc":"Initializers provide a way to centralize any RabbitMQ setup required by your application. In that sense, it should be started BEFORE any of your producers or consumers. Using an initializer, you can automatically setup queues, exchanges and bindings with simple keyword lists. defmodule MyInitializer do use Rabbit.Initializer def start_link(opts \\\\ []) do Rabbit.Initializer.start_link(__MODULE__, opts, name: __MODULE__) end # Callbacks @impl Rabbit.Initializer def init(:initializer, opts) do # Perform runtime config {:ok, opts} end end MyInitializer.start_link( connection: MyConnection, queues: [ [name: &quot;my_queue_1&quot;], [name: &quot;my_queue_2&quot;, durable: true], ], exchanges: [ [name: &quot;my_exchange_1&quot;], [name: &quot;my_exchange_2&quot;, type: :fanout, durable: true], ], bindings: [ [type: :queue, source: &quot;my_exchange_1&quot;, destination: &quot;my_queue_1&quot;, routing_key: &quot;my_key&quot;], [type: :exchange, source: &quot;my_exchange_2&quot;, destination: &quot;my_exchange_1&quot;] ] )"}]